@@ -341,18 +341,18 @@ local parsePerGlyphTable = function(offset, type, fd)
 end
 
 local parseMathVariants = function(offset, fd)
-  local parseGlyphAssembly = function(_offset, _fd)
-    local assembly = vstruct.read(">@".._offset.." italicsCorrection:{ &MathValueRecord } partCount:u2", _fd)
-    assembly.italicsCorrection = fetchMathValueRecord(assembly.italicsCorrection, _offset, _fd)
-    assembly.partRecords = vstruct.read("> "..assembly.partCount.."*{ &GlyphPartRecord }", _fd)
+  local parseGlyphAssembly = function(inner_offset, inner_fd)
+    local assembly = vstruct.read(">@"..inner_offset.." italicsCorrection:{ &MathValueRecord } partCount:u2", inner_fd)
+    assembly.italicsCorrection = fetchMathValueRecord(assembly.italicsCorrection, inner_offset, inner_fd)
+    assembly.partRecords = vstruct.read("> "..assembly.partCount.."*{ &GlyphPartRecord }", inner_fd)
     assembly.partCount = nil
     return assembly
   end
-  local parseMathGlyphConstruction = function(_offset, _fd)
-    local construction = vstruct.read(">@".._offset.." glyphAssemblyOffset:u2 variantCount:u2", _fd)
-    local mathGlyphVariantRecord = vstruct.read("> "..construction.variantCount.."*{ &MathGlyphVariantRecord }", _fd)
+  local parseMathGlyphConstruction = function(inner_offset, inner_fd)
+    local construction = vstruct.read(">@"..inner_offset.." glyphAssemblyOffset:u2 variantCount:u2", inner_fd)
+    local mathGlyphVariantRecord = vstruct.read("> "..construction.variantCount.."*{ &MathGlyphVariantRecord }", inner_fd)
     return {
-      glyphAssembly = construction.glyphAssemblyOffset ~= 0 and parseGlyphAssembly(_offset + construction.glyphAssemblyOffset, _fd) or nil,
+      glyphAssembly = construction.glyphAssemblyOffset ~= 0 and parseGlyphAssembly(inner_offset + construction.glyphAssemblyOffset, inner_fd) or nil,
       mathGlyphVariantRecord = mathGlyphVariantRecord
     }
   end
@@ -395,7 +395,7 @@ local function parseMath(s)
   if s:len() <= 0 then return end
   local fd = vstruct.cursor(s)
   local header = vstruct.read(">majorVersion:u2 minorVersion:u2 mathConstantsOffset:u2 mathGlyphInfoOffset:u2 mathVariantsOffset:u2", fd)
-  SU.debug("opentype-parser", "header = "..header)
+  SU.debug("opentype-parser", "header =", header)
   if header.majorVersion > 1 then return end
   vstruct.compile("MathValueRecord", "value:i2 deviceTableOffset:u2")
   vstruct.compile("RangeRecord", "startGlyphID:u2 endGlyphID:u2 startCoverageIndex:u2")
@@ -408,8 +408,8 @@ local function parseMath(s)
                                      " mathTopAccentAttachmentOffset:u2"..
                                      " extendedShapeCoverageOffset:u2"..
                                      " mathKernInfoOffset:u2", fd)
-  SU.debug("opentype-parser", "mathGlyphInfoOffset = "..header.mathGlyphInfoOffset)
-  SU.debug("opentype-parser", "mathGlyphInfo = "..mathGlyphInfo)
+  SU.debug("opentype-parser", "mathGlyphInfoOffset =", header.mathGlyphInfoOffset)
+  SU.debug("opentype-parser", "mathGlyphInfo =", mathGlyphInfo)
   local mathItalicsCorrection = parseIfPresent(header.mathGlyphInfoOffset, mathGlyphInfo.mathItalicsCorrectionInfoOffset, function(offset)
     return parsePerGlyphTable(offset, "&MathValueRecord", fd)
   end)
@@ -445,17 +445,28 @@ local function parsePost(s)
   }
 end
 
+local function parseOs2(s)
+  if s:len() <= 0 then return end
+  local fd = vstruct.cursor(s)
+  local header = vstruct.read(">version:u2 xAvgCharWidth:i2 usWeightClass:u2 usWidthClass:u2 fsType:u2 ySubscriptXSize:i2 ySubscriptYSize:i2 ySubscriptXOffset:i2 ySubscriptYOffset:i2 ySuperscriptXSize:i2 ySuperscriptYSize:i2 ySuperscriptXOffset:i2 ySuperscriptYOffset:i2, yStrikeoutSize:i2 yStrikeoutPosition:i2", fd)
+  return {
+    yStrikeoutPosition = header.yStrikeoutPosition,
+    yStrikeoutSize = header.yStrikeoutSize,
+  }
+end
+
 local parseFont = function(face)
   if not face.font then
     local font = {}
-    font.head = parseHead(hb.get_table(face.data, face.index, "head"))
-    font.names = parseName(hb.get_table(face.data, face.index, "name"))
-    font.maxp = parseMaxp(hb.get_table(face.data, face.index, "maxp"))
-    font.colr = parseColr(hb.get_table(face.data, face.index, "COLR"))
-    font.cpal = parseCpal(hb.get_table(face.data, face.index, "CPAL"))
-    font.svg  = parseSvg(hb.get_table(face.data, face.index, "SVG"))
-    font.math = parseMath(hb.get_table(face.data, face.index, "MATH"))
-    font.post = parsePost(hb.get_table(face.data, face.index, "post"))
+    font.head = parseHead(hb.get_table(face, "head"))
+    font.names = parseName(hb.get_table(face, "name"))
+    font.maxp = parseMaxp(hb.get_table(face, "maxp"))
+    font.colr = parseColr(hb.get_table(face, "COLR"))
+    font.cpal = parseCpal(hb.get_table(face, "CPAL"))
+    font.svg  = parseSvg(hb.get_table(face, "SVG"))
+    font.math = parseMath(hb.get_table(face, "MATH"))
+    font.post = parsePost(hb.get_table(face, "post"))
+    font.os2 = parseOs2(hb.get_table(face, "OS/2"))
     face.font = font
   end
   return face.font
@@ -476,7 +487,7 @@ local getSVG = function(face, gid)
   if not face.font.svg then return end
   local item = face.font.svg[gid]
   if not item then return end
-  local str = hb.get_table(face.data, face.index, "SVG")
+  local str = hb.get_table(face, "SVG")
   local start = item.svgDocOffset+1
   local svg = str:sub(start, start + item.svgDocLength-1)
   if svg[1] == "\x1f" and svg[2] == "\x8b" then
