@@ -2,6 +2,13 @@ local icu = require("justenoughicu")
 
 local chardata = require("char-def")
 
+SILE.settings:declare({
+  parameter = "languages.fixedNbsp",
+  type = "boolean",
+  default = false,
+  help = "Whether to treat U+00A0 (NO-BREAK SPACE) as a fixed-width space"
+})
+
 SILE.nodeMakers.base = pl.class({
 
     _init = function (self, options)
@@ -15,7 +22,7 @@ SILE.nodeMakers.base = pl.class({
     makeToken = function (self)
       if #self.contents > 0 then
         coroutine.yield(SILE.shaper:formNnode(self.contents, self.token, self.options))
-        SU.debug("tokenizer", "Token: "..self.token)
+        SU.debug("tokenizer", "Token:", self.token)
         self.contents = {}
         self.token = ""
         self.lastnode = "nnode"
@@ -43,6 +50,14 @@ SILE.nodeMakers.base = pl.class({
       self.lastnode = "penalty"
     end,
 
+    makeNonBreakingSpace = function (self)
+      -- Unicode Line Breaking Algorithm (UAX 14) specifies that U+00A0
+      -- (NO-BREAK SPACE) is expanded or compressed like a normal space.
+      coroutine.yield(SILE.nodefactory.kern(SILE.shaper:measureSpace(self.options)))
+      self.lastnode = "glue"
+      self.lasttype = "sp"
+    end,
+
     iterator = function (_, _)
       SU.error("Abstract function nodemaker:iterator called", true)
     end,
@@ -53,30 +68,44 @@ SILE.nodeMakers.base = pl.class({
       return chardata[cp]
     end,
 
-    isPunctuation = function (self, char)
-      return self.isPunctuationType[self:charData(char).category]
-    end,
-
-    isSpace = function (self, char)
-      return self.isSpaceType[self:charData(char).linebreak]
+    isActiveNonBreakingSpace = function (self, char)
+      return self:isNonBreakingSpace(char) and not SILE.settings:get("languages.fixedNbsp")
     end,
 
     isBreaking = function (self, char)
-      return self.isBreakingType[self:charData(char).linebreak]
+      return self.breakingTypes[self:charData(char).linebreak]
     end,
+
+    isNonBreakingSpace = function (self, char)
+      local c = self:charData(char)
+      return c.contextname and c.contextname == "nobreakspace"
+    end,
+
+    isPunctuation = function (self, char)
+      return self.puctuationTypes[self:charData(char).category]
+    end,
+
+    isSpace = function (self, char)
+      return self.spaceTypes[self:charData(char).linebreak]
+    end,
+
     isQuote = function (self, char)
-      return self.isQuoteType[self:charData(char).linebreak]
-    end
+      return self.quoteTypes[self:charData(char).linebreak]
+    end,
+
+    isWord = function (self, char)
+      return self.wordTypes[self:charData(char).linebreak]
+    end,
 
   })
 
 SILE.nodeMakers.unicode = pl.class(SILE.nodeMakers.base)
 
-SILE.nodeMakers.unicode.isWordType = { cm = true }
-SILE.nodeMakers.unicode.isSpaceType = { sp = true }
-SILE.nodeMakers.unicode.isBreakingType = { ba = true, zw = true }
-SILE.nodeMakers.unicode.isPunctuationType = { po = true }
-SILE.nodeMakers.unicode.isQuoteType = {} -- quote linebreak category is ambiguous depending on the language
+SILE.nodeMakers.unicode.breakingTypes = { ba = true, zw = true }
+SILE.nodeMakers.unicode.puctuationTypes = { po = true }
+SILE.nodeMakers.unicode.quoteTypes = {} -- quote linebreak category is ambiguous depending on the language
+SILE.nodeMakers.unicode.spaceTypes = { sp = true }
+SILE.nodeMakers.unicode.wordTypes = { cm = true }
 
 function SILE.nodeMakers.unicode:dealWith (item)
   local char = item.text
@@ -85,6 +114,9 @@ function SILE.nodeMakers.unicode:dealWith (item)
   if self:isSpace(item.text) then
     self:makeToken()
     self:makeGlue(item)
+  elseif self:isActiveNonBreakingSpace(item.text) then
+    self:makeToken()
+    self:makeNonBreakingSpace()
   elseif self:isBreaking(item.text) then
     self:addToken(char, item)
     self:makeToken()
@@ -92,7 +124,7 @@ function SILE.nodeMakers.unicode:dealWith (item)
   elseif self:isQuote(item.text) then
     self:addToken(char, item)
     self:makeToken()
-  elseif self.lasttype and (thistype and thistype ~= self.lasttype and not self.isWordType[thistype]) then
+  elseif self.lasttype and (thistype and thistype ~= self.lasttype and not self:isWord(thistype)) then
     self:addToken(char, item)
   else
     self:letterspace()
@@ -116,7 +148,7 @@ function SILE.nodeMakers.unicode:letterspace ()
   if self.token then self:makeToken() end
   if self.lastnode and self.lastnode ~= "glue" then
     local w = SILE.settings:get("document.letterspaceglue").width
-    SU.debug("tokenizer", "Letter space glue: " .. tostring(w))
+    SU.debug("tokenizer", "Letter space glue:", w)
     coroutine.yield(SILE.nodefactory.kern({ width = w }))
     self.lastnode = "glue"
     self.lasttype = "sp"
@@ -152,7 +184,11 @@ function SILE.nodeMakers.unicode:handleWordBreak (item)
   if self:isSpace(item.text) then
     -- Spacing word break
     self:makeGlue(item)
-  else -- a word break which isn't a space
+  elseif self:isActiveNonBreakingSpace(item.text) then
+    -- Non-breaking space word break
+    self:makeNonBreakingSpace()
+  else
+     -- a word break which isn't a space
     self:addToken(item.text, item)
   end
 end
@@ -161,7 +197,7 @@ function SILE.nodeMakers.unicode:handleLineBreak (item, subtype)
   -- Because we are in charge of paragraphing, we
   -- will override space-type line breaks, and treat
   -- them just as ordinary word spaces.
-  if self:isSpace(item.text) then
+  if self:isSpace(item.text) or self:isActiveNonBreakingSpace(item.text) then
     self:handleWordBreak(item)
     return
   end
